;
; Startup code for ՂՂ
;
;	by A.Watanabe
;

	.export		_exit

	.import		_main
	.import		_nmi_main
	.import		_irq_main

	.import		ppuinit			;S
	.import		paletteinit
	.import		_waitvblank


	; Linker generated symbols
	.import		__RAM_START__, __RAM_SIZE__
	.import		__SRAM_START__, __SRAM_SIZE__
	.import		__ROM0_START__, __ROM0_SIZE__
	.import		__STARTUP_LOAD__,__STARTUP_RUN__, __STARTUP_SIZE__
	.import		__CODE_LOAD__,__CODE_RUN__, __CODE_SIZE__
	.import		__RODATA_LOAD__,__RODATA_RUN__, __RODATA_SIZE__

	.include	"gekioko.inc"

; ------------------------------------------------------------------------
; 16 bytes INES header

.segment        "HEADER"

;    +--------+------+------------------------------------------+
;    | Offset | Size | Content(s)                               |
;    +--------+------+------------------------------------------+
;    |   0    |  3   | 'NES'                                    |
;    |   3    |  1   | $1A                                      |
;    |   4    |  1   | 16K PRG-ROM page count                   |
;    |   5    |  1   | 8K CHR-ROM page count                    |
;    |   6    |  1   | ROM Control Byte #1                      |
;    |        |      |   %####vTsM                              |
;    |        |      |    |  ||||+- 0=Horizontal mirroring      |
;    |        |      |    |  ||||   1=Vertical mirroring        |
;    |        |      |    |  |||+-- 1=SRAM enabled              |
;    |        |      |    |  ||+--- 1=512-byte trainer present  |
;    |        |      |    |  |+---- 1=Four-screen mirroring     |
;    |        |      |    |  |                                  |
;    |        |      |    +--+----- Mapper # (lower 4-bits)     |
;    |   7    |  1   | ROM Control Byte #2                      |
;    |        |      |   %####0000                              |
;    |        |      |    |  |                                  |
;    |        |      |    +--+----- Mapper # (upper 4-bits)     |
;    |  8-15  |  8   | $00                                      |
;    | 16-..  |      | Actual 16K PRG-ROM pages (in linear      |
;    |  ...   |      | order). If a trainer exists, it precedes |
;    |  ...   |      | the first PRG-ROM page.                  |
;    | ..-EOF |      | CHR-ROM pages (in ascending order).      |
;    +--------+------+------------------------------------------+

        .byte   $4e,$45,$53,$1a	; "NES"^Z
	.byte   2	        ; ines prg  - Specifies the number of 16k prg banks.
	.byte   1               ; ines chr  - Specifies the number of 8k chr banks.
	.byte   %00000001       ; ines mir  - Specifies VRAM mirroring of the banks.
	.byte   %00000000       ; ines map  - Specifies the NES mapper used.
	.byte   0,0,0,0,0,0,0,0	; 8 zeroes


; ------------------------------------------------------------------------
; Place the startup code in a special segment.

.segment	"STARTUP"

start:

; setup the CPU and System and Sound Driver nsd.lib

	sei
	cld

	lda	#%00000000
	sta	PPU_CTRL1
	sta	PPU_CTRL2

@wait1:	lda	PPU_STATUS
	bpl	@wait1
@wait2:	lda	PPU_STATUS
	bpl	@wait2
@wait3:	lda	PPU_STATUS
	bpl	@wait3

; Clear the WRAM

	lda	#0
	ldx	#$FF
	txs
	inx
@L0:
	sta	$0000,x
	sta	$0100,x
	sta	$0200,x
	sta	$0300,x
	sta	$0400,x
	sta	$0500,x
	sta	$0600,x
	sta	$0700,x

;	sta	$6000,x
;	sta	$6100,x
;	sta	$6200,x
;	sta	$6300,x
;	sta	$6400,x
;	sta	$6500,x
;	sta	$6600,x
;	sta	$6700,x
;	sta	$6800,x
;	sta	$6900,x
;	sta	$6A00,x
;	sta	$6B00,x
;	sta	$6C00,x
;	sta	$6D00,x
;	sta	$6E00,x
;	sta	$6F00,x
;
;	sta	$7000,x
;	sta	$7100,x
;	sta	$7200,x
;	sta	$7300,x
;	sta	$7400,x
;	sta	$7500,x
;	sta	$7600,x
;	sta	$7700,x
;	sta	$7800,x
;	sta	$7900,x
;	sta	$7A00,x
;	sta	$7B00,x
;	sta	$7C00,x
;	sta	$7D00,x
;	sta	$7E00,x
;	sta	$7F00,x

	inx
	bne	@L0

; Call initialize sound driver

	jsr	_nsd_init

; Call module constructors

	jsr	ppuinit
	jsr	paletteinit

; Push arguments and call main()

	jsr	_main

; Call module destructors. This is also the _exit entry.

_exit:

; Reset the NES

	jmp	start

; ------------------------------------------------------------------------
; hardware vectors
; ------------------------------------------------------------------------

.segment "VECTORS"

	.word	_nmi_main	; $fffa vblank nmi
	.word	start		; $fffc reset
	.word	_irq_main	; $fffe irq / brk

; ------------------------------------------------------------------------
; character data
; ------------------------------------------------------------------------

.segment "CHARS"

	.incbin		"gekioko.chr"


